@@ -32,6 +32,7 @@
 # THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 from adi.context_manager import context_manager
+from adi.jesd import jesd
 from adi.obs import obs
 from adi.rx_tx import rx_tx
 
@@ -45,8 +46,16 @@ class ad9371(rx_tx, context_manager):
     _obs_channel_names = ["voltage0_i", "voltage0_q"]
     _device_name = ""
 
-    def __init__(self, uri=""):
-
+    def __init__(
+        self, uri="", username="root", password="analog", disable_jesd_control=False
+    ):
+        """Initialize AD9371 interface class.
+        Args:
+            uri (str): URI of target platform with AD9371
+            username (str): SSH username for target board. Required for JESD monitoring
+            password (str): SSH password for target board. Required for JESD monitoring
+            disable_jesd_control (bool): Disable JESD status monitoring over SSH
+        """
         context_manager.__init__(self, uri, self._device_name)
 
         self._ctrl = self._ctx.find_device("ad9371-phy")
@@ -54,10 +63,23 @@ class ad9371(rx_tx, context_manager):
         self._rxobs = self._ctx.find_device("axi-ad9371-rx-obs-hpc")
         self._txdac = self._ctx.find_device("axi-ad9371-tx-hpc")
 
+        if not disable_jesd_control and jesd:
+            self._jesd = jesd(uri, username=username, password=password)
+
         rx_tx.__init__(self)
 
         self.obs = obs(self._ctx, self._rxobs, self._obs_channel_names)
 
+    @property
+    def ensm_mode(self):
+        """ensm_mode: Enable State Machine State Allows real time control over
+        the current state of the device. Options are: radio_on, radio_off"""
+        return self._get_iio_dev_attr_str("ensm_mode")
+
+    @ensm_mode.setter
+    def ensm_mode(self, value):
+        self._set_iio_dev_attr_str("ensm_mode", value)
+
     @property
     def gain_control_mode(self):
         """gain_control_mode: Mode of receive path AGC. Options are:
@@ -68,6 +90,24 @@ class ad9371(rx_tx, context_manager):
     def gain_control_mode(self, value):
         self._set_iio_attr("voltage0", "gain_control_mode", False, value)
 
+    @property
+    def rx_quadrature_tracking_en_chan0(self):
+        """Enable Quadrature tracking calibration for RX1"""
+        return self._get_iio_attr("voltage0", "quadrature_tracking_en", False)
+
+    @rx_quadrature_tracking_en_chan0.setter
+    def rx_quadrature_tracking_en_chan0(self, value):
+        self._set_iio_attr("voltage0", "quadrature_tracking_en", False, value)
+
+    @property
+    def rx_quadrature_tracking_en_chan1(self):
+        """Enable Quadrature tracking calibration for RX2"""
+        return self._get_iio_attr("voltage1", "quadrature_tracking_en", False)
+
+    @rx_quadrature_tracking_en_chan1.setter
+    def rx_quadrature_tracking_en_chan1(self, value):
+        self._set_iio_attr("voltage1", "quadrature_tracking_en", False, value)
+
     @property
     def rx_hardwaregain_chan0(self):
         """rx_hardwaregain: Gain applied to RX path channel 0. Only applicable when
@@ -90,6 +130,42 @@ class ad9371(rx_tx, context_manager):
         if self.gain_control_mode == "manual":
             self._set_iio_attr("voltage1", "hardwaregain", False, value)
 
+    @property
+    def rx_temp_comp_gain_chan0(self):
+        """rx_temp_comp_gain_chan0: """
+        return self._get_iio_attr("voltage0", "temp_comp_gain", False)
+
+    @rx_temp_comp_gain_chan0.setter
+    def rx_temp_comp_gain_chan0(self, value):
+        self._set_iio_attr("voltage0", "temp_comp_gain", False, value)
+
+    @property
+    def rx_temp_comp_gain_chan1(self):
+        """rx_temp_comp_gain_chan1: """
+        return self._get_iio_attr("voltage1", "temp_comp_gain", False)
+
+    @rx_temp_comp_gain_chan1.setter
+    def rx_temp_comp_gain_chan1(self, value):
+        self._set_iio_attr("voltage1", "temp_comp_gain", False, value)
+
+    @property
+    def tx_quadrature_tracking_en_chan0(self):
+        """Enable Quadrature tracking calibration for TX1"""
+        return self._get_iio_attr("voltage0", "quadrature_tracking_en", True)
+
+    @tx_quadrature_tracking_en_chan0.setter
+    def tx_quadrature_tracking_en_chan0(self, value):
+        self._set_iio_attr("voltage0", "quadrature_tracking_en", True, value)
+
+    @property
+    def tx_quadrature_tracking_en_chan1(self):
+        """Enable Quadrature tracking calibration for TX2"""
+        return self._get_iio_attr("voltage1", "quadrature_tracking_en", True)
+
+    @tx_quadrature_tracking_en_chan1.setter
+    def tx_quadrature_tracking_en_chan1(self, value):
+        self._set_iio_attr("voltage1", "quadrature_tracking_en", True, value)
+
     @property
     def tx_hardwaregain_chan0(self):
         """tx_hardwaregain: Attenuation applied to TX path channel 0"""
@@ -168,6 +244,14 @@ class ad9371(rx_tx, context_manager):
         dec = 8 if self.rx_enable_dec8 else 1
         return self._get_iio_attr("voltage0", "sampling_frequency", False) / dec
 
+    @property
+    def orx_sample_rate(self):
+        """orx_sample_rate: Sample rate ORX path in samples per second
+            This value will reflect the correct value when 8x decimator is enabled
+        """
+        dec = 8 if self.rx_enable_dec8 else 1
+        return self._get_iio_attr("voltage2", "sampling_frequency", False) / dec
+
     @property
     def tx_sample_rate(self):
         """tx_sample_rate: Sample rate TX path in samples per second
@@ -203,6 +287,45 @@ class ad9371(rx_tx, context_manager):
     def sn_lo(self, value):
         self._set_iio_attr("altvoltage2", "RX_SN_LO_frequency", True, value)
 
+    @property
+    def obs_gain_control_mode(self):
+        """obs_gain_control_mode: Mode of Obs/Sniffer receive path AGC. Options are:
+        automatic, hybrid, manual"""
+        return self._get_iio_attr_str("voltage2", "gain_control_mode", False)
+
+    @obs_gain_control_mode.setter
+    def obs_gain_control_mode(self, value):
+        self._set_iio_attr("voltage2", "gain_control_mode", False, value)
+
+    @property
+    def obs_hardwaregain(self):
+        """obs_hardwaregain: Gain applied to Obs/Sniffer receive path chan0. Only applicable when
+        obs_gain_control_mode is set to 'manual'"""
+        return self._get_iio_attr("voltage2", "hardwaregain", False)
+
+    @obs_hardwaregain.setter
+    def obs_hardwaregain(self, value):
+        if self.obs_gain_control_mode == "manual":
+            self._set_iio_attr("voltage2", "hardwaregain", False, value)
+
+    @property
+    def obs_temp_comp_gain(self):
+        """obs_temp_comp_gain: """
+        return self._get_iio_attr("voltage2", "temp_comp_gain", False)
+
+    @obs_temp_comp_gain.setter
+    def obs_temp_comp_gain(self, value):
+        self._set_iio_attr("voltage2", "temp_comp_gain", False, value)
+
+    @property
+    def obs_quadrature_tracking_en(self):
+        """Enable Quadrature tracking calibration for OBS chan0"""
+        return self._get_iio_attr("voltage2", "quadrature_tracking_en", False)
+
+    @obs_quadrature_tracking_en.setter
+    def obs_quadrature_tracking_en(self, value):
+        self._set_iio_attr("voltage2", "quadrature_tracking_en", False, value)
+
     @property
     def obs_rf_port_select(self):
         """obs_rf_port_select: Observation path source. Options are:
@@ -224,3 +347,7 @@ class ad9371(rx_tx, context_manager):
     @obs_rf_port_select.setter
     def obs_rf_port_select(self, value):
         self._set_iio_attr("voltage2", "rf_port_select", False, value)
+
+    @property
+    def jesd204_statuses(self):
+        return self._jesd.get_all_statuses()
